
<h3>Navigation</h3>
<p>
  <a href="{{ pathto(master_doc) }}">Documentation overview</a>
</p>
<ul>
  {%- for parent in parent %}
    <li><a href="{{ parent.link|e }}">{{ parent.title }}</a>
    <ul>
  {%- endfor %}
  {%- if prev %}
    <li>Previous: <a
      href="{{ prev.link|e }}"
      title="{{ _('previous chapter') }}"
    >{{ prev.title }}</a>
    </li>
  {%- endif %}
  {%- if next %}
    <li>Next: <a
      href="{{ next.link|e }}"
      title="{{ _('next chapter') }}"
    >{{ next.title }}</a>
    </li>
  {%- endif %}
  {%- for parent in parents %}
    </ul></li>
  {%- endfor %}
</ul>

